/**
 * Copyright (c) 2022, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { AccessControlConstants, Show } from "@wso2is/access-control";
import { ContentLoader, Heading, LinkButton, Text } from "@wso2is/react-components";
import cloneDeep from "lodash-es/cloneDeep";
import React, { CSSProperties, FC, ReactElement, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useSelector } from "react-redux";
import { Card, Grid } from "semantic-ui-react";
import { CategoryItem } from "./app-category-item";
import {
    ApplicationTemplateListItemInterface,
    ApplicationTemplateManagementUtils
} from "../../../../features/applications";
import { getApplicationTemplateIllustrations } from "../../../../features/applications/configs";
import CustomApplicationTemplate
    from "../../../../features/applications/data/application-templates/templates/custom-application/custom-application.json";
import {
    AppConstants,
    AppState,
    EventPublisher,
    history
} from "../../../../features/core";

type ApplicationIntegrationCardPropsInterface = {
    isApplicationsFetchRequestLoading?: boolean;
    isApplicationsAvailable?: boolean;
    onTemplateSelected: (group: ApplicationTemplateListItemInterface) => void;
};

/**
 * Shows a categorized set of application templates.
 * @param props {ApplicationIntegrationCardPropsInterface}
 * @constructor
 */
export const ApplicationIntegrationCard: FC<ApplicationIntegrationCardPropsInterface> = (
    props: ApplicationIntegrationCardPropsInterface
): ReactElement => {

    const {
        isApplicationsFetchRequestLoading,
        isApplicationsAvailable,
        onTemplateSelected
    } = props;

    const { t } = useTranslation();

    const applicationTemplates: ApplicationTemplateListItemInterface[] =
        useSelector((state: AppState) => state?.application?.groupedTemplates);
    const [ isLoadingTemplates, setIsLoadingTemplates ] = useState<boolean>(false);

    const eventPublisher: EventPublisher = EventPublisher.getInstance();

    /**
     * Fetch the application templates if
     * {@link applicationTemplates} is unavailable.
     */
    useEffect(() => {
        if (applicationTemplates === undefined) {
            setIsLoadingTemplates(true);
            ApplicationTemplateManagementUtils.getApplicationTemplates().finally(() => {
                setIsLoadingTemplates(false);
            });
        }
    }, [ applicationTemplates ]);

    const handleTemplateSelection = (templateId: string): void => {

        if (!applicationTemplates) return;

        let selected: ApplicationTemplateListItemInterface =
            applicationTemplates.find(({ templateId: matcher }) => matcher === templateId);

        if (!selected) return;

        if (templateId === CustomApplicationTemplate.id) {
            selected = cloneDeep(CustomApplicationTemplate);
        }

        eventPublisher.publish("application-click-create-new", {
            type: selected.templateId
        });

        onTemplateSelected(selected);
    };

    const cardContentStyles = (): CSSProperties => ({
        height: "100%",
        paddingBottom: "10px",
        paddingTop: "12px"
    });

    return (
        <Card
            data-componentid="application-integration-card"
            data-testid="application-integration-card"
            fluid
            className="basic-card no-hover getting-started-card"
        >
            <Card.Content extra className="description-container mb-0 pb-0">
                <div className="card-heading">
                    <Heading as="h2">Integrate your application</Heading>
                    <Text muted className="mb-0 pb-0">Choose application type</Text>
                </div>
            </Card.Content>
            <Card.Content
                extra
                className="description-container"
                style={ cardContentStyles() }>
                {
                    isLoadingTemplates
                        ? <ContentLoader/>
                        : (
                            <Grid>
                                <Grid.Row columns={ 2 }>
                                    <Grid.Column width={ 8 }>
                                        <CategoryItem
                                            onClick={ () => {
                                                handleTemplateSelection(
                                                    "single-page-application"
                                                );
                                            } }
                                            iconSize="tiny"
                                            techStackIcons={ [
                                                getApplicationTemplateIllustrations().spa
                                            ] }
                                            categoryName="Single-Page Application"/>
                                    </Grid.Column>
                                    <Grid.Column width={ 8 }>
                                        <CategoryItem
                                            onClick={ () => {
                                                handleTemplateSelection(
                                                    "traditional-web-application"
                                                );
                                            } }
                                            iconSize="tiny"
                                            techStackIcons={ [
                                                getApplicationTemplateIllustrations().oidcWebApp
                                            ] }
                                            categoryName="Traditional Web Application"/>
                                    </Grid.Column>
                                </Grid.Row>
                                <Grid.Row columns={ 2 }>
                                    <Grid.Column width={ 8 }>
                                        <CategoryItem
                                            comingSoon
                                            onClick={ () => void 0 }
                                            iconSize="tiny"
                                            techStackIcons={ [
                                                getApplicationTemplateIllustrations().oidcMobile
                                            ] }
                                            categoryName="Mobile Application"/>
                                    </Grid.Column>
                                    <Grid.Column width={ 8 }>
                                        <CategoryItem
                                            onClick={ () => {
                                                handleTemplateSelection(
                                                    "custom-application"
                                                );
                                            } }
                                            iconSize="tiny"
                                            techStackIcons={ [
                                                getApplicationTemplateIllustrations().customApp
                                            ] }
                                            categoryName="Standard-Based Application"/>
                                    </Grid.Column>
                                </Grid.Row>
                            </Grid>
                        )
                }
            </Card.Content>
            <Card.Content extra className="action-container">
                <Grid>
                    <Grid.Row columns={ 1 }>
                        <Grid.Column width={ 16 }>
                            <LinkButton
                                compact
                                style={ { textAlign: "left" } }
                                loading={ isApplicationsFetchRequestLoading }
                                data-testid="getting-started-page-add-app-button"
                                data-componentid="getting-started-page-add-app-button"
                                className="quick-start-action"
                                onClick={ () => {
                                    eventPublisher.publish("console-getting-started-add-application-path");
                                    history.push({
                                        pathname: isApplicationsAvailable
                                            ? AppConstants.getPaths().get("APPLICATIONS")
                                            : AppConstants.getPaths().get("APPLICATION_TEMPLATES")
                                    });
                                } }
                            >
                                <span
                                    className="button-text"
                                    data-componentid="getting-started-page-add-app-button-text"
                                >
                                    <Show
                                        when={ AccessControlConstants.APPLICATION_WRITE }
                                        fallback={
                                            t("extensions:common.quickStart." +
                                                "sections.integrateApps.actions.view")
                                        }>
                                        {
                                            isApplicationsAvailable
                                                ? t("extensions:common.quickStart" +
                                                    ".sections.integrateApps" +
                                                    ".actions.manage")
                                                : "See More"
                                        }
                                    </Show>
                                </span>
                            </LinkButton>
                        </Grid.Column>
                    </Grid.Row>
                </Grid>
            </Card.Content>
        </Card>
    );

};

/**
 * Default props of {@link ApplicationIntegrationCard}
 */
ApplicationIntegrationCard.defaultProps = {
    isApplicationsAvailable: false,
    isApplicationsFetchRequestLoading: false
};
